@@ -26,8 +26,9 @@
 # ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
-from __future__ import absolute_import
-import distutils.version
+from __future__ import absolute_import, division, print_function
+__metaclass__ = type
+
 import os
 import signal
 import threading
@@ -43,52 +44,9 @@ import ansible_mitogen.loaders
 import ansible_mitogen.mixins
 import ansible_mitogen.process
 
-import ansible
 import ansible.executor.process.worker
-
-try:
-    # 2.8+ has a standardized "unset" object.
-    from ansible.utils.sentinel import Sentinel
-except ImportError:
-    Sentinel = None
-
-
-ANSIBLE_VERSION_MIN = (2, 3)
-ANSIBLE_VERSION_MAX = (2, 9)
-NEW_VERSION_MSG = (
-    "Your Ansible version (%s) is too recent. The most recent version\n"
-    "supported by Mitogen for Ansible is %s.x. Please check the Mitogen\n"
-    "release notes to see if a new version is available, otherwise\n"
-    "subscribe to the corresponding GitHub issue to be notified when\n"
-    "support becomes available.\n"
-    "\n"
-    "    https://mitogen.rtfd.io/en/latest/changelog.html\n"
-    "    https://github.com/dw/mitogen/issues/\n"
-)
-OLD_VERSION_MSG = (
-    "Your version of Ansible (%s) is too old. The oldest version supported by "
-    "Mitogen for Ansible is %s."
-)
-
-
-def _assert_supported_release():
-    """
-    Throw AnsibleError with a descriptive message in case of being loaded into
-    an unsupported Ansible release.
-    """
-    v = ansible.__version__
-    if not isinstance(v, tuple):
-        v = tuple(distutils.version.LooseVersion(v).version)
-
-    if v[:2] < ANSIBLE_VERSION_MIN:
-        raise ansible.errors.AnsibleError(
-            OLD_VERSION_MSG % (v, ANSIBLE_VERSION_MIN)
-        )
-
-    if v[:2] > ANSIBLE_VERSION_MAX:
-        raise ansible.errors.AnsibleError(
-            NEW_VERSION_MSG % (ansible.__version__, ANSIBLE_VERSION_MAX)
-        )
+import ansible.template
+import ansible.utils.sentinel
 
 
 def _patch_awx_callback():
@@ -99,12 +57,11 @@ def _patch_awx_callback():
     # AWX uses sitecustomize.py to force-load this package. If it exists, we're
     # running under AWX.
     try:
-        from awx_display_callback.events import EventContext
-        from awx_display_callback.events import event_context
+        import awx_display_callback.events
     except ImportError:
         return
 
-    if hasattr(EventContext(), '_local'):
+    if hasattr(awx_display_callback.events.EventContext(), '_local'):
         # Patched version.
         return
 
@@ -113,8 +70,8 @@ def _patch_awx_callback():
         ctx = tls.setdefault('_ctx', {})
         ctx.update(kwargs)
 
-    EventContext._local = threading.local()
-    EventContext.add_local = patch_add_local
+    awx_display_callback.events.EventContext._local = threading.local()
+    awx_display_callback.events.EventContext.add_local = patch_add_local
 
 _patch_awx_callback()
 
@@ -132,8 +89,7 @@ def wrap_action_loader__get(name, *args, **kwargs):
     get_kwargs = {'class_only': True}
     if name in ('fetch',):
         name = 'mitogen_' + name
-    if ansible.__version__ >= '2.8':
-        get_kwargs['collection_list'] = kwargs.pop('collection_list', None)
+    get_kwargs['collection_list'] = kwargs.pop('collection_list', None)
 
     klass = ansible_mitogen.loaders.action_loader__get(name, **get_kwargs)
     if klass:
@@ -153,6 +109,7 @@ REDIRECTED_CONNECTION_PLUGINS = (
     'lxc',
     'lxd',
     'machinectl',
+    'podman',
     'setns',
     'ssh',
 )
@@ -217,7 +174,7 @@ class AnsibleWrappers(object):
         with references to the real functions.
         """
         ansible_mitogen.loaders.action_loader.get = wrap_action_loader__get
-        ansible_mitogen.loaders.connection_loader.get = wrap_connection_loader__get
+        ansible_mitogen.loaders.connection_loader.get_with_context = wrap_connection_loader__get
 
         global worker__run
         worker__run = ansible.executor.process.worker.WorkerProcess.run
@@ -230,7 +187,7 @@ class AnsibleWrappers(object):
         ansible_mitogen.loaders.action_loader.get = (
             ansible_mitogen.loaders.action_loader__get
         )
-        ansible_mitogen.loaders.connection_loader.get = (
+        ansible_mitogen.loaders.connection_loader.get_with_context = (
             ansible_mitogen.loaders.connection_loader__get
         )
         ansible.executor.process.worker.WorkerProcess.run = worker__run
@@ -324,7 +281,7 @@ class StrategyMixin(object):
             name=task.action,
             class_only=True,
         )
-        if play_context.connection is not Sentinel:
+        if play_context.connection is not ansible.utils.sentinel.Sentinel:
             # 2.8 appears to defer computing this until inside the worker.
             # TODO: figure out where it has moved.
             ansible_mitogen.loaders.connection_loader.get(
@@ -352,7 +309,6 @@ class StrategyMixin(object):
         Wrap :meth:`run` to ensure requisite infrastructure and modifications
         are configured for the duration of the call.
         """
-        _assert_supported_release()
         wrappers = AnsibleWrappers()
         self._worker_model = self._get_worker_model()
         ansible_mitogen.process.set_worker_model(self._worker_model)
@@ -371,3 +327,44 @@ class StrategyMixin(object):
                 self._worker_model.on_strategy_complete()
         finally:
             ansible_mitogen.process.set_worker_model(None)
+
+    def _smuggle_to_connection_reset(self, task, play_context, iterator, target_host):
+        """
+        Create a templar and make it available for use in Connection.reset().
+        This allows templated connection variables to be used when Mitogen
+        reconstructs its connection stack.
+        """
+        variables = self._variable_manager.get_vars(
+            play=iterator._play, host=target_host, task=task,
+            _hosts=self._hosts_cache, _hosts_all=self._hosts_cache_all,
+        )
+        templar = ansible.template.Templar(
+            loader=self._loader, variables=variables,
+        )
+
+        # Required for remote_user option set by variable (e.g. ansible_user).
+        # Without it remote_user in ansible.cfg gets used.
+        play_context = play_context.set_task_and_variable_override(
+            task=task, variables=variables, templar=templar,
+        )
+        play_context.post_validate(templar=templar)
+
+        # Required for timeout option set by variable (e.g. ansible_timeout).
+        # Without it the task timeout keyword (default: 0) gets used.
+        play_context.update_vars(variables)
+
+        # Stash the task and templar somewhere Connection.reset() can find it
+        play_context.vars.update({
+            '_mitogen.smuggled.reset_connection': (task, templar),
+        })
+        return play_context
+
+    def _execute_meta(self, task, play_context, iterator, target_host):
+        if task.args['_raw_params'] == 'reset_connection':
+            play_context = self._smuggle_to_connection_reset(
+                task, play_context, iterator, target_host,
+            )
+
+        return super(StrategyMixin, self)._execute_meta(
+            task, play_context, iterator, target_host,
+        )
